//! The example can be run by this command
//! `cargo run --example concat`

use tabled::{object::Segment, Alignment, Concat, ModifyObject, Style, Table, Tabled};

#[derive(Debug, Tabled)]
struct Weather {
    temperature_c: f64,
    wind_ms: f64,
}

impl Weather {
    fn new(temperature_c: f64, wind_ms: f64) -> Self {
        Self {
            temperature_c,
            wind_ms,
        }
    }
}

#[derive(Debug, Tabled)]
struct Location(
    #[tabled(rename = "latitude")] f64,
    #[tabled(rename = "longitude")] f64,
);

fn main() {
    let weather_data = [
        Weather::new(16.0, 3000.0),
        Weather::new(-20.0, 300.0),
        Weather::new(40.0, 100.0),
    ];

    let location_data = [
        Location(111.111, 333.333),
        Location(5.111, 7282.1),
        Location(0.0, 0.0),
        Location(0.0, 0.0),
    ];

    let weather_table = Table::new(weather_data);
    let location_table = Table::new(location_data);

    let data_table = weather_table
        .with(Concat::horizontal(location_table))
        .with(Style::empty())
        .with(Segment::all().modify().with(Alignment::left()));

    println!("{}", data_table);
}
